import { reactionSchema } from '@/models/reaction';
import { apiPath } from '@/redux/apiPath';
import { axiosInstance } from '@/services/api/utils/axiosInstance';
import { Reaction } from '@/types/models';
import { getErrorMessage } from '@/utils/getErrorMessage';
import { validateDataUsingZodSchema } from '@/utils/validateDataUsingZodSchema';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { z } from 'zod';
import { ThunkConfig } from '@/types/store';
import { REACTIONS_FETCHING_ERROR_PREFIX } from './reactions.constants';

export const getReactionsByIds = createAsyncThunk<Reaction[] | undefined, number[], ThunkConfig>(
  'reactions/getByIds',
  async (ids, { rejectWithValue }) => {
    try {
      const response = await axiosInstance.get<Reaction[]>(apiPath.getReactionsWithIds(ids));
      const isDataValid = validateDataUsingZodSchema(response.data, z.array(reactionSchema));

      if (!isDataValid) {
        return undefined;
      }

      return response.data;
    } catch (error) {
      const errorMessage = getErrorMessage({ error, prefix: REACTIONS_FETCHING_ERROR_PREFIX });
      return rejectWithValue(errorMessage);
    }
  },
);
