import { getMultiBioEntity } from '@/redux/bioEntity/bioEntity.thunks';
import { getMultiChemicals } from '@/redux/chemicals/chemicals.thunks';
import { getMultiDrugs } from '@/redux/drugs/drugs.thunks';
import { PerfectMultiSearchParams } from '@/types/search';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { getErrorMessage } from '@/utils/getErrorMessage';
import { ThunkConfig } from '@/types/store';
import type { RootState } from '../store';
import { dispatchPluginsEvents } from './search.thunks.utils';
import { DATA_SEARCHING_ERROR_PREFIX } from './search.constants';

type GetSearchDataProps = PerfectMultiSearchParams;

export const getSearchData = createAsyncThunk<
  void,
  GetSearchDataProps,
  { state: RootState } & ThunkConfig
>(
  'project/getSearchData',
  // eslint-disable-next-line consistent-return
  async ({ searchQueries, isPerfectMatch }, { dispatch, getState, rejectWithValue }) => {
    try {
      await Promise.all([
        dispatch(getMultiBioEntity({ searchQueries, isPerfectMatch })),
        dispatch(getMultiDrugs(searchQueries)),
        dispatch(getMultiChemicals(searchQueries)),
      ]);

      dispatchPluginsEvents(searchQueries, getState());
    } catch (error) {
      const errorMessage = getErrorMessage({ error, prefix: DATA_SEARCHING_ERROR_PREFIX });

      return rejectWithValue(errorMessage);
    }
  },
);
